import { ApplicationDataSource } from '../database/datasource.js';
import { LtiMessageModel, PlatformModel } from '../database/model.js';
import { handle_login_request, verifyMessage } from '../lti/message.js';
import { getPlatformEndpoints } from '../lti/platforms/index.js';
import { LTIMessage } from './lti_message.js';
import { LTIResource } from './lti_resource.js';
import { LTISession } from './lti_session.js';
export class LTIPlatform {
    platform_model;
    platform_id;
    // #region Initializers
    constructor(platform_model) {
        this.platform_model = platform_model;
        this.platform_id = this.platform_model.id;
    }
    static async create(associated_user) {
        const _platform = ApplicationDataSource.manager.create(PlatformModel, {
            associated_user,
        });
        await ApplicationDataSource.manager.insert(PlatformModel, _platform);
        return new LTIPlatform(_platform);
    }
    static async list() {
        const _platforms = await ApplicationDataSource.manager.find(PlatformModel, {
            where: { registrated: true },
        });
        return _platforms.map(platform => new LTIPlatform(platform));
    }
    static async byId({ platform_id }) {
        const platform = await ApplicationDataSource.manager.findOneByOrFail(PlatformModel, {
            id: platform_id,
        });
        return new LTIPlatform(platform);
    }
    // #endregion
    async completeRegistration(params) {
        const _platform = {
            id: this.platform_model.id,
            iss: params.iss,
            client_id: params.client_id,
            deployment_id: params.lti_deployment_id,
            authentication_request_url: params.authentication_request_url,
            access_token_url: params.access_token_url,
            jwks_url: params.jwks_url,
            registrated: true,
        };
        this.platform_model = await ApplicationDataSource.manager.save(PlatformModel, _platform);
    }
    async getAuthUrl(params) {
        if (!this.platform_model.registrated) {
            const urls = await getPlatformEndpoints(params);
            await this.completeRegistration({ ...params, ...urls });
        }
        return await handle_login_request(params, this.platform_model);
    }
    async verifyLtiMessage(params) {
        // Get the messageModel from the database (needed for jwks_url and nonce)
        const message = await ApplicationDataSource.manager.findOneOrFail(LtiMessageModel, {
            where: { id: params.state },
            relations: ['platform'],
        });
        try {
            // Verify the message
            const payload = await verifyMessage(params.id_token, message.nonce, this.platform_model.jwks_url);
            // Return the verified message
            return new LTIMessage(payload);
        }
        finally {
            // delete it to prevent replay attacks
            await ApplicationDataSource.manager.delete(LtiMessageModel, message.id);
        }
    }
    async startSession(params) {
        const payload = await this.verifyLtiMessage(params);
        const resource = await LTIResource.getOrCreate(payload, this);
        return await LTISession.createSession(payload, resource);
    }
    async delete() {
        await ApplicationDataSource.manager.delete(PlatformModel, this.platform_model);
    }
}
//# sourceMappingURL=lti_platform.js.map