import { logger } from '@crosslab/service-common';
import { ApplicationDataSource } from '../database/datasource.js';
import { LtiResourceModel, LtiResourceStudent, LtiResourceStudentRoleMapModel } from '../database/model.js';
import { membership } from '../lti/membership.js';
export class LTIResourceStudent {
    resource_id;
    student_model;
    role_mapping_models;
    student_id;
    external_id;
    // #region Initializers
    constructor(resource_id, student_model, role_mapping_models) {
        this.resource_id = resource_id;
        this.student_model = student_model;
        this.role_mapping_models = role_mapping_models;
        this.student_id = student_model.id;
        this.external_id = student_model.external_id;
    }
    static async newStudent(resource_id, student) {
        const role_mappings = await ApplicationDataSource.manager.findBy(LtiResourceStudentRoleMapModel, { student });
        return new LTIResourceStudent(resource_id, student, role_mappings);
    }
    static async byId({ student_id, resource_id }) {
        const student = await ApplicationDataSource.manager.findOneByOrFail(LtiResourceStudent, {
            id: student_id,
            resource: { id: resource_id },
        });
        return this.newStudent(resource_id, student);
    }
    static async list({ resource_id }) {
        const students = await ApplicationDataSource.manager.findBy(LtiResourceStudent, {
            resource: { id: resource_id },
        });
        return Promise.all(students.map(student => this.newStudent(resource_id, student)));
    }
    // #endregion
    get email() {
        return this.student_model.email;
    }
    get name() {
        return this.student_model.name;
    }
    get role_mappings() {
        return this.role_mapping_models.map(role => ({
            role: role.role,
            device: role.device,
        }));
    }
    async set_role_mapping(p) {
        if (p.device === 'GROUP') {
            await ApplicationDataSource.manager.delete(LtiResourceStudentRoleMapModel, {
                student: { id: this.student_id },
                role: p.role,
            });
        }
        else {
            await ApplicationDataSource.manager.upsert(LtiResourceStudentRoleMapModel, {
                role: p.role,
                student: { id: this.student_id },
                device: p.device,
            }, ['student', 'role']);
        }
    }
    static async updateStudentsFromLtiResource(resource_id) {
        const resource_model = await ApplicationDataSource.manager.findOneByOrFail(LtiResourceModel, {
            id: resource_id,
        });
        logger.info(`Updating students for resource ${resource_id}`);
        logger.info(resource_model.namesServiceUrl);
        if (!resource_model.namesServiceUrl) {
            return; // No name service provided
        }
        const members = await membership(resource_model.platform, resource_model.namesServiceUrl);
        const students = members.map(m => ({
            name: m.name ?? `${m.given_name} ${m.family_name}`,
            external_id: m.user_id,
            resource: { id: resource_id },
        }));
        logger.info(`Updating students for resource ${resource_id}`);
        logger.info(students);
        await ApplicationDataSource.createQueryBuilder(LtiResourceStudent, 'student').insert().values(students).orUpdate(['name'], ['resourceId', 'external_id']).execute();
    }
}
//# sourceMappingURL=lti_resource_student.js.map