import { logger } from '@crosslab/service-common';
import { clients } from '../clients/index.js';
import { ApplicationDataSource } from '../database/datasource.js';
import { LtiResourceStudent, LtiResourceStudentRoleMapModel, LtiSessionModel, LtiSessionRoleMapModel } from '../database/model.js';
import { random } from '../helper/generators.js';
import { LTIMessage } from './lti_message.js';
export class LTISession {
    session_model;
    role_mapping_models;
    session_id;
    resource_id;
    launchMessage;
    // #region Initializers
    constructor(session_model, role_mapping_models) {
        this.session_model = session_model;
        this.role_mapping_models = role_mapping_models;
        this.session_id = session_model.id;
        this.resource_id = session_model.resource.id;
        this.launchMessage = new LTIMessage(session_model.launchMessage);
    }
    static async createSession(message, resource) {
        const id = random();
        let session = ApplicationDataSource.manager.create(LtiSessionModel, {
            id,
            resource: { id: resource.resource_id },
            launchMessage: message.raw,
        });
        await ApplicationDataSource.manager.insert(LtiSessionModel, session);
        session = await ApplicationDataSource.manager.findOneByOrFail(LtiSessionModel, {
            id,
        });
        if (message.context_memberships_url)
            resource.setNameService(message.context_memberships_url);
        const student = await ApplicationDataSource.manager.findOne(LtiResourceStudent, {
            select: ['id'],
            where: { resource: { id: resource.resource_id }, external_id: message.user_id },
        });
        const role_mappings = student
            ? await ApplicationDataSource.manager.findBy(LtiResourceStudentRoleMapModel, {
                student: { id: student.id },
            })
            : [];
        await ApplicationDataSource.manager.insert(LtiSessionRoleMapModel, role_mappings.map(role => ({
            session: { id: session.id },
            role: role.role,
            device: role.device,
        })));
        const role_mappings_models = await ApplicationDataSource.manager.findBy(LtiSessionRoleMapModel, {
            session: { id: session.id },
        });
        return new LTISession(session, role_mappings_models);
    }
    static async byId(id) {
        const _session = await ApplicationDataSource.manager.findOneByOrFail(LtiSessionModel, { id: id.session_id });
        const role_mappings_models = await ApplicationDataSource.manager.findBy(LtiSessionRoleMapModel, {
            session: { id: id.session_id },
        });
        return new LTISession(_session, role_mappings_models);
    }
    // #endregion
    async createAccessToken() {
        if (!this.session_model.resource.platform.associated_user) {
            throw new Error('The platform does not have an associated user');
        }
        return await clients.authentication.createToken({
            user: this.session_model.resource.platform.associated_user,
            claims: { lti_token: true },
        });
    }
    async get_experiment_uri() {
        return this.session_model.experiment_uri;
    }
    async createOrUpdateExperiment({ experiment: experimentClient }) {
        experimentClient = clients.experiment;
        try {
            if (!this.session_model.resource.experiment_template_uri) {
                throw new Error('The resource does not have an experiment template');
            }
            const template = await experimentClient.getTemplate(this.session_model.resource.experiment_template_uri);
            logger.info('Creating experiment', template.configuration);
            const experiment = template.configuration;
            for (const { role, device } of this.role_mappings) {
                experiment.devices = experiment.devices.map(d => (d.role === role ? { ...d, device } : d));
            }
            if (this.session_model.experiment_uri) {
                await experimentClient.updateExperiment(this.session_model.experiment_uri, experiment);
            }
            else {
                this.session_model.experiment_uri = (await experimentClient.createExperiment({ status: 'created', ...experiment })).url;
                await ApplicationDataSource.manager.save(LtiSessionModel, this.session_model);
            }
        }
        catch (e) {
            logger.warn(e);
            this.session_model.experiment_uri = undefined;
            await ApplicationDataSource.manager.save(LtiSessionModel, this.session_model);
        }
    }
    get role_mappings() {
        return this.role_mapping_models.map(role => ({
            role: role.role,
            device: role.device,
        }));
    }
    async set_role_mapping(p) {
        if (p.device === 'GROUP') {
            await ApplicationDataSource.manager.delete(LtiSessionRoleMapModel, {
                session: { id: this.session_id },
                role: p.role,
            });
        }
        else {
            await ApplicationDataSource.manager.upsert(LtiSessionRoleMapModel, {
                role: p.role,
                session: { id: this.session_id },
                device: p.device,
            }, ['session', 'role']);
        }
        this.role_mapping_models = await ApplicationDataSource.manager.findBy(LtiSessionRoleMapModel, {
            session: { id: this.session_id },
        });
        await this.createOrUpdateExperiment(clients);
    }
}
//# sourceMappingURL=lti_session.js.map